import { debounce } from "@lincode/utils"
import { useEffect, useState } from "react"

export default (cb: (e: DragEvent) => void) => {
    const [dropEl, setDropEl] = useState<HTMLDivElement | null>()
    const [dragOver, setDragOver] = useState(false)

    useEffect(() => {
        if (!dropEl) return

        const setDragOverDebounced = debounce((value: boolean) => setDragOver(value), 100, "trailing")

        let valueOld = false
        const setDragOverTrack = (value: boolean) => {
            if (valueOld === value) return
            valueOld = value

            setDragOverDebounced(value)
        }

        const dragOverCb = (e: DragEvent) => {
            e.preventDefault()
            setDragOverTrack(true)
        }
        dropEl.addEventListener("dragover", dragOverCb)

        const dragEnterCb = (e: DragEvent) => {
            e.preventDefault()
            setDragOverTrack(true)
        }
        dropEl.addEventListener("dragenter", dragEnterCb)

        const dragLeaveCb = (e: DragEvent) => {
            e.preventDefault()
            setDragOverTrack(false)
        }
        dropEl.addEventListener("dragleave", dragLeaveCb)
        
        const dropCb = (e: DragEvent) => {
            e.preventDefault()
            setDragOverTrack(false)
            cb(e)
        }
        dropEl.addEventListener("drop", dropCb)

        return () => {
            dropEl.removeEventListener("dragover", dragOverCb)
            dropEl.removeEventListener("dragenter", dragEnterCb)
            dropEl.removeEventListener("dragleave", dragLeaveCb)
            dropEl.removeEventListener("drop", dropCb)
        }
    }, [dropEl])

    return <const>[setDropEl, dragOver]
}