import { useFileDrop } from "."

const getAllFileEntries = async (dataTransferItemList?: DataTransferItemList) => {
    if (!dataTransferItemList) return []

	const fileEntries: Array<any> = []
	const queue: Array<any> = []
	for (let i = 0; i < dataTransferItemList.length; i++)
		queue.push(dataTransferItemList[i].webkitGetAsEntry())

	while (queue.length > 0) {
		const entry = queue.shift()
		if (entry.isFile)
			fileEntries.push(entry)
		else if (entry.isDirectory) {
            const entries = await readAllDirectoryEntries(entry.createReader())
			queue.push(...entries)
        }
	}
	return fileEntries
}

const readAllDirectoryEntries = async (directoryReader: any) => {
	const entries: Array<any> = []
	let readEntries = await readEntriesPromise(directoryReader)
	while (readEntries.length > 0) {
		entries.push(...readEntries)
		readEntries = await readEntriesPromise(directoryReader)
	}
	return entries
}

const readEntriesPromise = (directoryReader: any) => {
    return new Promise<Array<any>>((resolve, reject) => {
        directoryReader.readEntries(resolve, reject)
    })
}

const getFile = (fileEntry: any) => new Promise<File>((resolve, reject) => fileEntry.file(resolve, reject))

export default (cb: (e: DragEvent, files: Array<File>) => void) => {
    const [setDropEl, dragOver] = useFileDrop(async e => {
        const fileEntries = await getAllFileEntries(e.dataTransfer?.items)
        const files = await Promise.all(fileEntries.map(getFile))
        cb(e, files)
    })

    return [setDropEl, dragOver]
}